.TH WEBSTER 3 "October 18, 1990" "NeXT Computer, Inc."
.UC 4
.SH NAME
.nh
referenceOpen, referenceClose, nextDefinition, getDefinition, 
getNextDefinition, seekEntry, useFullText, freeDefinition, putDefinition \- 
programmatic interface to the Merriam-Webster database
.SH SYNOPSIS
.ft L
#include <text/webster.h>
.br
.SH DESCRIPTION
.nh
These functions provide a programmatic interface to the Merriam-Webster 
database used by \fBWebster(1)\fR. 
They are part of the common text library, libtext. 
.sp
The following program fragment prints a simple, 
unformatted definition for a word on the standard output
.in +.5in
.sp
.na
.nf
\#include \<text/webster.h\>

void
main()
{
	ReferenceBook *book;
	Definition *d;
	int exact = 1;

	book = referenceOpen("Webster-Dictionary");
	dictionaryFoldSenses = 80; /* fold output lines to 80 chars */
	if (d = getDefinition("amphora", book, exact))
	{
		putDefinition(d, 0);
		printf("\n");
		freeDefinition(d);
	}
}
.fi
.in -.5in
.sp
.PP
.nh
The database includes the \fBNinth Collegiate Dictionary (tm)\fR
and the \fBCollegiate Thesaurus (tm)\fR.
These books are found in
\fB/NextLibrary/References/Webster-Dictionary\fP, and 
\fB/NextLibrary/References/Webster-Thesaurus\fP, respectively.

Each book consists of a BTreeFile containing an entry BTree and an index BTree;
the dictionary also may contain a \fBfull-index\fR BTree.
There may also be artwork in a \fBpictures\fR subdirectory, and some ancillary
material in an \fBinfo\fR subdirectory, 
corresponding to the `front matter' of the paper book.

The conventions documented here are subject to change as multilingual dictionary formats emerge.
The entry BTree stores the source, one record for each entry in the book;
it is keyed by a unique number called the entryID.
The index BTrees store index records keyed by word;
each index record contains a list of entryIDs.

An entry is stored as a sequential list of tagged fields. 
A tag is a one byte code describing an element within an entry 
(\fIe.g.\fR, a function, a pronunciation, \fIetc.\fR). 
The formatting of the data is type-specific.
A series of one or two byte codes marking special glyphs
and font changes is also used. 

The three principal data structures are \fISenseList\fR, 
which is a linked list of the elements in an entry; 
\fIDefinition\fR, which describes a complete entry;
and \fIReferenceBook\fR, which is a descriptor holding BTreeCursors 
and other material describing a reference book.
.in +.5in
.sp
.na
.nf
typedef struct SenseList
// A SenseList is a list of tagged fields: s[0] is the tag, s[1] is the data.
{
	char *s;
	int n;
	struct SenseList *next;
} SenseList;

typedef struct
/* A Definition contains a number of fields, all
optional. The field pointers are taken from the SenseList,
and point to the first of 0 or more fields of the appropriate
type which may be present in the entry. */
{
	char *entry;		// the main entry word
	char *dotted;		// the dotted form of the word
	char *pronunciation;	// the pronunciation field
	char *function;		// the grammatical function
	char *date;		// date word was first used
	char *etymology;	// etymological description
	char *inflection;	// inflected forms (-ed, -ies, -ing, etc.)
	char *variant;		// variant forms and inflections
	char *picture;		// pathname of file containing picture
	char *subject;		// internal subject codes, per-sense
	SenseList *l;		// linked list comprising the entry
	int section;		// dMain, dAbbr, dForeign, dBio, dGeo, etc.
	entryID ID;		// ID of entry in book
	struct ReferenceBook *book;	// what book it came from
} Definition;

/* Each entry in a book has a unique identifier called an entryID.
Queries return lists of IDs in the entryList member of ReferenceBook.
entryIDs can be selected using the seekEntry function. */
typedef int entryID;

typedef struct ReferenceBook
/* A reference book includes a source file, an index,
and possibly a full-text index. The client's state is
described by this data structure, 
which may include pointers to book-specific i/o routines. */
{
	BTreeFile		*source;	// BTreeFile containing the book
	BTreeCursor	*entryCursor;
	BTreeCursor	*indexCursor;
	BTreeCursor	*fullIndexCursor;
	entryID		*entryList;
	int			entryListNum;
	int			entryListCount;
	Definition	*(*getDef)();
	int			(*putDef)();
	int			(*freeDef)();
} ReferenceBook;
.in -.5in
.fi
.sp
.PP
.SH SUMMARY
.nh
ReferenceBook *\fBreferenceOpen\fR(char *\fIname\fR);
.in +.5in
.sp
Opens reference book \fIname\fR for searching 
and returns a pointer to an open descriptor, or zero in the event of failure.
Currently, \fIname\fR must be either "Webster-Dictionary" or "Webster-Thesaurus".
.in -.5in
.PP
.nh
int \fBreferenceClose\fR(ReferenceBook *\fIr\fR);
.in +.5in
.sp
Closes \fIr\fR, and frees the associated storage.
.in -.5in
.PP
.nh
Definition *\fBnextDefinition\fR(ReferenceBook *\fIbook\fR);
.in +.5in
.sp
Returns the next definition in \fIbook\fR.
Consecutive calls step through the entries in the book.
.in -.5in
.PP
.nh
Definition *\fBgetDefinition\fR(char *\fIword\fR, 
.br
.in +1in
ReferenceBook *\fIbook\fR, int \fImatch\fR);
.in -1in
.in +.5in
.sp
Gets the first definition for \fIword\fR in \fIbook\fR, 
and returns a pointer to the open descriptor, 
or zero if no definition was found.
If \fImatch\fR is true, the word must match exactly, otherwise
prefix matches are accepted.
.in -.5in
.PP
.nh
Definition *\fBgetNextDefinition\fR(char *\fIword\fR, 
.br
.in +1in
ReferenceBook *\fIbook\fR, int \fImatch\fR);
.in -1in
.in +.5in
.sp
Returns the next definition matching \fIword\fR, or zero if no there are more.
This routine may be called after an initial call to \fIgetDefinition()\fR.
.in -.5in
.PP
.nh
Definition *\fBseekEntry\fR(entryID \fIitsID\fR, ReferenceBook *\fIbook\fR);
.in +.5in
.sp
Returns the definition identified by \fIitsID\fR, 
if such a definition exists.
This function may be used to walk through 
the lists of entries returned in the entryList member of a ReferenceBook.
.in -.5in
.PP
.nh
int \fBuseFullText\fR(ReferenceBook *\fIbook\fR, int \fIyesOrNo\fR);
.in +.5in
.sp
Selects or deselects the full text index, 
according to the value of \fIyesOrNo\fR.
Returns 1 if the request was satisfied.
.in -.5in
.PP
.nh
int \fBfreeDefinition\fR(Definition *\fId\fR);
.in +.5in
.sp
Frees the storage used by \fId\fR.
.in -.5in
.PP
.nh
int
\fBputDefinition\fR(Definition *\fId\fR, int (*\fIoutput\fR)());
.in +.5in
.sp
Generates the definition \fId\fR, calling the function \fIoutput\fR
for each character processed.
The information is preconverted at lookup time according 
to the value of the global variable \fBdictionaryOutputFormat\fR.
Currently, this may be one of W_ASCII, W_PS, W_TROFF or W_VERBOSE.
If no output function is given, 
the information is written to the standard output.

The W_ASCII output format is the default, 
as it is likely to be the most useful 
for computational linguistic applications. 
When W_ASCII is used, the global variable \fBdictionaryFoldSenses\fR 
should be set to the desired line width in characters.
.in -.5in
.SH "SEE ALSO"
text(3)
